@@ -7,7 +7,7 @@ package Spreadsheet::WriteExcel::Formula;
 #
 # Used in conjunction with Spreadsheet::WriteExcel
 #
-# Copyright 2000-2008, John McNamara, jmcnamara@cpan.org
+# Copyright 2000-2010, John McNamara, jmcnamara@cpan.org
 #
 # Documentation after __END__
 #
@@ -24,7 +24,7 @@ use Carp;
 use vars qw($VERSION @ISA);
 @ISA = qw(Exporter);
 
-$VERSION = '2.22';
+$VERSION = '2.39';
 
 ###############################################################################
 #
@@ -58,6 +58,7 @@ sub new {
                     _ext_sheets     => {},
                     _ext_refs       => {},
                     _ext_ref_count  => 0,
+                    _ext_names      => {},
                  };
 
     bless $self, $class;
@@ -136,6 +137,7 @@ sub _init_parser {
                         | ref2d
                         | ref3d
                         | function
+                        | name
                         | '(' expr ')'  { [$item[2], 'ptgParen'] }
 
         # Match a string.
@@ -177,11 +179,15 @@ sub _init_parser {
 
         # Match a function name.
         function:         /[A-Z0-9-_.]+/ '()'
-                        { ['_func', $item[1]] }
+                        { ['_funcV', $item[1]] }
                         | /[A-Z0-9-_.]+/ '(' expr ')'
-                        { ['_class', $item[1], $item[3], '_func', $item[1]] }
+                        { ['_class', $item[1], $item[3], '_funcV', $item[1]] }
                         | /[A-Z0-9-_.]+/ '(' list ')'
-                        { ['_class', $item[1], $item[3], '_func', $item[1]] }
+                        { ['_class', $item[1], $item[3], '_funcV', $item[1]] }
+
+        # Match a defined name.
+        name:           /[A-Za-z_]\w+/
+                        { ['_name', $item[1]] }
 
         # Boolean values.
         true:           'TRUE'  { [ 'ptgBool', 1 ] }
@@ -331,12 +337,26 @@ sub parse_tokens {
             $token      = shift @_;
             $parse_str .= $self->_convert_range3d($token, $class);
         }
-        elsif ($token eq '_func') {
+        elsif ($token =~ /^_name/) {
+            ($modifier  = $token) =~ s/_name//;
+            $class      = $class[-1];
+            $class      = 0 if $modifier eq 'R';
+            $class      = 1 if $modifier eq 'V';
+            $token = shift @_;
+            $parse_str .= $self->_convert_name($token, $class);
+        }
+        elsif ($token eq '_funcV') {
             $token = shift @_;
             $parse_str .= $self->_convert_function($token, $num_args);
             pop @class;
             $num_args = 0; # Reset after use
         }
+        elsif ($token eq '_func') {
+            $token = shift @_;
+            $parse_str .= $self->_convert_function($token, $num_args, 1);
+            pop @class;
+            $num_args = 0; # Reset after use
+        }
         elsif (exists $ptg{$token}) {
             $parse_str .= pack("C", $ptg{$token});
         }
@@ -434,7 +454,7 @@ sub _check_volatile {
 
     for my $i (0..@tokens-1) {
         # If the next token is a function check if it is volatile.
-        if ($tokens[$i] eq '_func' and $functions{$tokens[$i+1]}[3]) {
+        if ($tokens[$i] =~ m/^_func/ and $functions{$tokens[$i+1]}[3]) {
             $volatile = 1;
             last;
         }
@@ -795,25 +815,10 @@ sub set_ext_sheets {
     my $worksheet   = shift;
     my $index       = shift;
 
-    #my $ref         = "$index:$index";
-
     # The _ext_sheets hash is used to translate between worksheet names
     # and their index
     $self->{_ext_sheets}->{$worksheet} = $index;
 
-
-    # 2D sheet refs such as '=Sheet1:Sheet2!A1' can only be added after all
-    # worksheets have been added.
-    #return 0 if $index < $self->{_ext_ref_count}; TODO
-
-
-    # The _ext_refs hash is used to correlate the external references used in
-    # formulas with the index stored in the Workbook EXTERNSHEET record.
-    #$self->{_ext_refs}->{$ref} = $index;
-    #$self->{_ext_ref_count}++;
-
-    # No errors
-    #return 1;
 }
 
 
@@ -821,26 +826,14 @@ sub set_ext_sheets {
 #
 # get_ext_sheets()
 #
-# This semi-public method is used to update the hash of sheet names. It is
-# updated by the add_worksheet() method of the Workbook class.
-#
-# TODO
+# This semi-public method is used to get the worksheet references that were
+# used in formulas for inclusion in the EXTERNSHEET Workbook record.
 #
 sub get_ext_sheets {
 
     my $self  = shift;
 
-    # TODO
-    my %refs = %{$self->{_ext_refs}};
-    return %refs;
-
-    #my @refs = sort {$refs{$a} <=> $refs{$b}} keys %refs;
-
-    #foreach my $ref (@refs) {
-    #    $ref = [split /:/, $ref];
-    #}
-
-    #return @refs;
+    return %{$self->{_ext_refs}};
 }
 
 
@@ -848,7 +841,7 @@ sub get_ext_sheets {
 #
 # get_ext_ref_count()
 #
-# TODO This semi-public method is used to update the hash of sheet names. It is
+# This semi-public method is used to update the hash of sheet names. It is
 # updated by the add_worksheet() method of the Workbook class.
 #
 sub get_ext_ref_count {
@@ -859,11 +852,48 @@ sub get_ext_ref_count {
 }
 
 
+###############################################################################
+#
+# _get_name_index()
+#
+# Look up the index that corresponds to an external defined name. The hash of
+# defined names is updated by the define_name() method in the Workbook class.
+#
+sub _get_name_index {
+
+    my $self        = shift;
+    my $name        = shift;
+
+    if (not exists $self->{_ext_names}->{$name}) {
+        die "Unknown defined name $name in formula\n";
+    }
+    else {
+        return $self->{_ext_names}->{$name};
+    }
+}
+
+
+###############################################################################
+#
+# set_ext_name()
+#
+# This semi-public method is used to update the hash of defined names.
+#
+sub set_ext_name {
+
+    my $self        = shift;
+    my $name        = shift;
+    my $index       = shift;
+
+    $self->{_ext_names}->{$name} = $index;
+}
+
+
 ###############################################################################
 #
 # _convert_function()
 #
-# Convert a function to a ptgFunc or ptgFuncVarV depending on the number of
+# Convert a function to a ptgFuncV or ptgFuncVarV depending on the number of
 # args that it takes.
 #
 sub _convert_function {
@@ -871,6 +901,7 @@ sub _convert_function {
     my $self     = shift;
     my $token    = shift;
     my $num_args = shift;
+    my $non_var  = shift;
 
     die "Unknown function $token() in formula\n"
         unless defined $functions{$token}[0];
@@ -884,17 +915,61 @@ sub _convert_function {
             die "Incorrect number of arguments for $token() in formula\n";
         }
         else {
-            return pack("Cv", $ptg{ptgFuncV}, $functions{$token}[0]);
+            if ($non_var) {
+                return pack("Cv", $ptg{ptgFunc},  $functions{$token}[0]);
+            }
+            else {
+                return pack("Cv", $ptg{ptgFuncV}, $functions{$token}[0]);
+            }
         }
     }
 
     # Variable number of args eg. SUM($i,$j,$k, ..).
     if ($args == -1) {
-        return pack "CCv", $ptg{ptgFuncVarV}, $num_args, $functions{$token}[0];
+        if ($non_var) {
+            return pack "CCv", $ptg{ptgFuncVar},
+                                $num_args, $functions{$token}[0];
+        }
+        else {
+            return pack "CCv", $ptg{ptgFuncVarV},
+                                $num_args, $functions{$token}[0];
+        }
     }
 }
 
 
+###############################################################################
+#
+# _convert_name()
+#
+# Convert a symbolic name into a name reference.
+#
+sub _convert_name {
+
+    my $self     = shift;
+    my $name     = shift;
+    my $class    = shift;
+
+    my $ptgName;
+
+    my $name_index = $self->_get_name_index($name);
+
+    # The ptg value depends on the class of the ptg.
+    if    ($class == 0) {
+        $ptgName = $ptg{ptgName};
+    }
+    elsif ($class == 1) {
+        $ptgName = $ptg{ptgNameV};
+    }
+    elsif ($class == 2) {
+        $ptgName = $ptg{ptgNameA};
+    }
+
+
+    return pack 'CV', $ptgName, $name_index;
+}
+
+
 ###############################################################################
 #
 # _cell_to_rowcol($cell_ref)
@@ -1322,6 +1397,7 @@ sub _initialize_hashes {
 
 __END__
 
+=encoding latin1
 
 =head1 NAME
 
@@ -1392,14 +1468,14 @@ The additional tokens are:
     _ref3d      The next token is a 3d cell reference
     _range2d    The next token is a 2d range
     _range3d    The next token is a 3d range
-    _func       The next token is a function
+    _funcV       The next token is a function
     _arg        The next token is the number of args for a function
     _class      The next token is a function name
     _vol        The formula contains a voltile function
 
 The C<_arg> token is generated for all lists but is only used for functions that take a variable number of arguments.
 
-The C<_class> token indicates the start of the arguments to a function. This allows the post-processor to decide the "class" of the ref and range arguments that the function takes. The class can be reference, value or array. Since function calls can be nested, the class variable is stored on a stack in the C<@class> array. The class of the ref or range is then read as the top element of the stack C<$class[-1]>. When a C<_func> is read it pops the class value.
+The C<_class> token indicates the start of the arguments to a function. This allows the post-processor to decide the "class" of the ref and range arguments that the function takes. The class can be reference, value or array. Since function calls can be nested, the class variable is stored on a stack in the C<@class> array. The class of the ref or range is then read as the top element of the stack C<$class[-1]>. When a C<_funcV> is read it pops the class value.
 
 Certain Excel functions such as RAND() and NOW() are designated as volatile and must be recalculated by Excel every time that a cell is updated. Any formulas that contain one of these functions has a specially formatted C<ptgAttr> tag prepended to it to indicate that it is volatile.
 
@@ -1424,6 +1500,6 @@ John McNamara jmcnamara@cpan.org
 
 =head1 COPYRIGHT
 
- MM-MMVIII, John McNamara.
+ MM-MMX, John McNamara.
 
 All Rights Reserved. This module is free software. It may be used, redistributed and/or modified under the same terms as Perl itself.
